@@ -1,6 +1,6 @@
 ##----------------------------------------------------------------------------
 ## A real Try Catch Block Implementation Using Perl Filter - ~/lib/Nice/Try.pm
-## Version v1.0.7
+## Version v1.1.0
 ## Copyright(c) 2021 DEGUEST Pte. Ltd.
 ## Author: Jacques Deguest <jack@deguest.jp>
 ## Created 2020/05/17
@@ -25,7 +25,7 @@ BEGIN
     use Scalar::Util ();
     use List::Util ();
     use Want ();
-    our $VERSION = 'v1.0.7';
+    our $VERSION = 'v1.1.0';
     our $ERROR;
     our( $CATCH, $DIED, $EXCEPTION, $FINALLY, $HAS_CATCH, @RETVAL, $SENTINEL, $TRY, $WANTARRAY );
 }
@@ -857,12 +857,27 @@ EOT
                     # $self->_messagef( 3, "Catch assignment is: '%s'", $cdef->{var}->content );
                     # my $str = $cdef->{var}->content;
                     my $str = $self->_serialize( $cdef->{var} );
-                    $str =~ s/^\([[:blank:]]*|[[:blank:]]*\)$//g;
-                    if( $str =~ /^(\S+)[[:blank:]]+(\$\S+)$/ )
+                    $str =~ s/^\([[:blank:]\h]*|[[:blank:]]*\)$//g;
+                    # My::Exception $e
+                    if( $str =~ /^(\S+)[[:blank:]\h]+(\$\S+)$/ )
                     {
-                        my( $ex_class, $ex_var ) = ( $1, $2 );
-                        $cdef->{class} = $ex_class;
-                        $cdef->{var} = $ex_var;
+                        @$cdef{qw( class var )} = ( $1, $2 );
+                    }
+                    elsif( $str =~ /^(\S+)[[:blank:]\h]+(\$\S+)[[:blank:]\h]+where[[:blank:]\h]+\{(.*?)\}$/ )
+                    {
+                        @$cdef{qw( class var where )} = ( $1, $2, $3 );
+                    }
+                    elsif( $str =~ /^(\$\S+)[[:blank:]\h]+where[[:blank:]\h]+\{(.*?)\}$/ )
+                    {
+                        @$cdef{qw( var where )} = ( $1, $2 );
+                    }
+                    elsif( $str =~ /^(\$\S+)[[:blank:]\h]+isa[[:blank:]\h]+(\S+)(?:[[:blank:]\h]+where[[:blank:]\h]+\{(.*?)\})?$/ )
+                    {
+                        @$cdef{qw( var class where )} = ( $1, $2, $3 );
+                    }
+                    elsif( $str =~ /^(?<var>\$\S+)[[:blank:]\h]+isa[[:blank:]\h]*\([[:blank:]\h]*(?<quote>["'])?(?<class>[^[:blank:]\h\'\"\)]+)\k{quote}[[:blank:]\h]*\)(?:[[:blank:]\h]+where[[:blank:]\h]+\{(?<where>.*?)\})?$/ )
+                    {
+                        @$cdef{qw( var class where )} = ( $+{var}, $+{class}, $+{where} );
                     }
                     else
                     {
@@ -873,6 +888,7 @@ EOT
                 {
                     # $self->_message( 3, "No Catch assignment found" );
                 }
+                
                 if( $cdef->{block} )
                 {
                     # $self->_messagef( 3, "Catch block is:\n%s", $cdef->{block}->content );
@@ -977,7 +993,20 @@ EOT
                 if( $cdef->{var} )
                 {
                     my $ex_var = $cdef->{var};
-                    if( $cdef->{class} )
+                    if( $cdef->{class} && $cdef->{where} )
+                    {
+                        my $ex_class = $cdef->{class};
+                        my $eval = "q{CORE::local \$_ = \$Nice::Try::EXCEPTION; my $ex_var = \$Nice::Try::EXCEPTION; CORE::local \$\@ = \$Nice::Try::EXCEPTION; $cdef->{where}}";
+                        $catch_section = <<EOT;
+        ${cond}( Scalar::Util::blessed( \$Nice::Try::EXCEPTION ) && \$Nice::Try::EXCEPTION->isa( '$ex_class' ) && eval( $eval ) )
+        {
+            CORE::local \$\@ = \$Nice::Try::EXCEPTION;
+            my $ex_var = \$Nice::Try::EXCEPTION;
+$catch_code
+        }
+EOT
+                    }
+                    elsif( $cdef->{class} )
                     {
                         my $ex_class = $cdef->{class};
                         # Tilmann Haeberle (TH) 2021-03-25: Fix: properly test for exception class inheritance via ->isa
@@ -988,6 +1017,18 @@ EOT
             my $ex_var = \$Nice::Try::EXCEPTION;
 $catch_code
         }
+EOT
+                    }
+                    elsif( $cdef->{where} )
+                    {
+                        my $eval = "q{CORE::local \$_ = \$Nice::Try::EXCEPTION; my $ex_var = \$Nice::Try::EXCEPTION; CORE::local \$\@ = \$Nice::Try::EXCEPTION; $cdef->{where}}";
+                        $catch_section = <<EOT;
+        ${cond}( eval( $eval ) )
+        {
+            CORE::local \$\@ = \$Nice::Try::EXCEPTION;
+            my $ex_var = \$Nice::Try::EXCEPTION;
+$catch_code
+        }
 EOT
                     }
                     # No class, just variable assignment like $e or something
@@ -1691,9 +1732,37 @@ Also since version 1.0.0, L<Nice::Try> is context aware:
     # scalar reference context
     my $name = ${$o->info};
 
+And you also have granual power in the catch block to filter which exception to handle:
+
+    try
+    {
+        die( Exception->new( "Arghhh" => 401 ) );
+    }
+    catch( $oopsie isa Exception where { $_->message =~ /Arghhh/ && $_->code == 500 } )
+    {
+        # Do something to deal with some server error
+    }
+    catch( $oopsie isa Exception where { $_->message =~ /Arghhh/ && $_->code == 401 } )
+    {
+        # should reach here.
+    }
+    catch( $oh_well isa("Exception") )
+    {
+        # Default using another way to filter by Exception
+    }
+    catch( $oopsie where { /Oh no/ } )
+    {
+        # Do something based on the value of a simple error; not an exception class
+    }
+    # Default
+    catch( $default )
+    {
+        print( "Unknown error: $default\n" );
+    }
+
 =head1 VERSION
 
-    v1.0.7
+    v1.1.0
 
 =head1 DESCRIPTION
 
@@ -2128,6 +2197,62 @@ Since, try-catch block can be nested, the following would work too:
         # do something about it
     }
 
+=head1 EXCEPTION CLASS
+
+As mentioned above, you can use class when raising exceptions and you can filter them in a variety of way since version 1.0.7 when you catch them.
+
+Here are your options:
+
+=over 4
+
+=item 1. catch( Exception::Class $error_variable ) { }
+
+=item 2. catch( Exception::Class $error_variable where { $condition } ) { }
+
+Here C<$condition> could be anything that fits in a legitimate perl's block, such as:
+
+    try
+    {
+        die( Exception->new( "Oh no!", { code => 401 } ) );
+    }
+    catch( Exception $oopsie where { $_->code >= 400 && $_->code < 499 })
+    {
+        # some more handling here
+    }
+
+In the condition block C<$_> will always be made available and will correspond to the exception object thrown, just like C<$oopsie> in this example. C<$@> is also availble with the exception object as its value.
+
+=item 3. catch( $e isa Exception::Class ) { }
+
+=item 4. catch( $e isa('Exception::Class') ) { }
+
+=item 5. catch( $e isa("Exception::Class") ) { }
+
+=item 6. catch( $e isa Exception::Class where { $condition } ) { }
+
+=item 7. catch( $e isa('Exception::Class') where { $condition } ) { }
+
+=item 8. catch( $e isa("Exception::Class") where { $condition } ) { }
+
+=item 9. catch( $e where { $condition } ) { }
+
+This is not a class execption catching, but worse mentioning. For example:
+
+    try
+    {
+        die( "Something bad happened.\n" );
+    }
+    catch( $e where { /something bad/i })
+    {
+        # Do something about it
+    }
+    catch( $e )
+    {
+        # Default here
+    }
+
+=back
+
 =head1 LOOPS
 
 Since version v0.2.0 L<Nice::Try> supports the use of flow control keywords such as C<next>, C<last> and C<redo> inside try-catch blocks. For example:
